/*  fgwirc - fungw based minimal irc client
    Copyright (C) 2019,2021  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page and contact: http://repo.hu/projects/fgwirc
*/

#ifndef IRC_H
#define IRC_H

#include <sockev/sockev.h>

#include "window.h"
#include "global_typedefs.h"

extern char *irc_ident, *irc_user, *irc_nick;
extern int irc_default_port;

typedef enum irc_state_e {
	IRCS_OFFLINE,
	IRCS_CONNECTING,
	IRCS_CONNECTED
} irc_state_t;

struct irc_server_s {
	char *hostname;
	int port;
	win_t *status_win;

	irc_state_t state;
	sockev_t sock;
	int sid;
	char nick[16];       /* own current nick on the current server */
	char want_nick[16];  /* preferred nickname */
};

/* parsed form of nick!ident@host */
struct irc_from_s {
	char nick[515];
	int nicklen;
	char ident[515];
	int identlen;
	char host[515];
	int hostlen;

	char copy[1030]; /* a copy of nick starting at [1] (template for outgoing messages) */
};


extern vtp0_t irc_servers;

/* Connect to a server; if sid is -1, pick and ID automatically; win will
   be the status window for the server */
int irc_connect_new(win_t *win, const char *hostname, int port, int sid);

void irc_reconnect(int sid);
void irc_disconnect(int sid);

sockev_t irc_server_sock(int sid);

int irc_send_win_msg(win_t *win, const char *str);
int irc_send_win_raw(win_t *win, const char *str);
int irc_send_all_raw(const char *str);

int irc_lookup_server(const char *name);

irc_server_t *irc_get_win_server(win_t *win);
const char *irc_get_win_my_nick(win_t *win, const char *fallback);

void irc_init(void);

#define irc_is_prefix_chan(p) (((p) == '&') || ((p) == '#') || ((p) == '+') || ((p) == '!'))

/* Deliver an incoming message (line) to the most appropriate window */
void irc_win_deliver_incoming(irc_server_t *server, const irc_from_t *fr, const char *target, const char *line);

#endif
