#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "buffer.h"

struct sockev_buff_s {
	char *from; /* to be sent from (within data) */
	int left;   /* length, calculated from from */
	sockev_buff_t *next; /* next element in the buff */
	char data[1];
};

void sockev_buffer_init(sockev_buffer_t *buffer)
{
	buffer->first = NULL;
	buffer->last = NULL;
	buffer->total = 0;
}

void sockev_buffer_append(sockev_buffer_t *buffer, const char *b, int len)
{
	sockev_buff_t *buff;

	if (len <= 0)
		return;

	buff = malloc(sizeof(sockev_buff_t) + len - 1);
	buff->from = buff->data;
	buff->left = len;
	memcpy(buff->data, b, len);

	buff->next = NULL;
	if (buffer->last != NULL)
		buffer->last->next = buff;
	buffer->last = buff;
	if (buffer->first == NULL)
		buffer->first = buff;

	buffer->total += len;
}

int sockev_buffer_get(sockev_buffer_t *buffer, const char **b, int *len)
{
	if (buffer->first == NULL)
		return -1;

	*b = buffer->first->from;
	*len = buffer->first->left;

	return 0;
}

/* after writing out a chunk (after call to sockev_buffer_get()), remove the chunk from the buffer */
void sockev_buffer_remove(sockev_buffer_t *buffer, int len)
{
	sockev_buff_t *buff;

	if ((buffer->first == NULL) || (len <= 0) || (buffer->first->left < len)) {
		fprintf(stderr, "sockev buffer: invalid buffer read %p %d\n", buffer->first, len);
		abort();
		return;
	}

	buff = buffer->first;
	buffer->total -= len;

	/* wrote the whole chunk? */
	if (buffer->first->left == len) {
		buffer->first = buff->next;
		if (buffer->first == NULL)
			buffer->last = NULL;
		free(buff);
		return;
	}

	/* partial write */
	buff->left -= len;
	buff->from += len;

	if (buffer->total < 0)
		fprintf(stderr, "sockev buffer: negative total: %d\n", buffer->total);
}

int sockev_buffer_total(sockev_buffer_t *buffer)
{
	return buffer->total;
}

void sockev_buffer_uninit(sockev_buffer_t *buffer)
{
	/* keep on removing head of the list until the list is empty */
	while(buffer->first != NULL)
		sockev_buffer_remove(buffer, buffer->first->left);

	if (buffer->total != 0)
		fprintf(stderr, "sockev buffer: non-zero total after uninit: %d\n", buffer->total);
}

